@@ -1,5 +1,5 @@
 /*
- * "$Id: ipp.c 6484 2007-04-30 17:18:21Z mike $"
+ * "$Id: ipp.c 6583 2007-06-20 22:07:55Z mike $"
  *
  *   IPP backend for the Common UNIX Printing System (CUPS).
  *
@@ -870,8 +870,8 @@ main(int  argc,				/* I - Number of command-line args */
           break;
 
       if (i < format_sup->num_values)
-        num_options = cupsAddOption("document-format", content_type,
-	                            num_options, &options);
+        ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_MIMETYPE,
+	             "document-format", NULL, content_type);
     }
 
     if (copies_sup && version > 0 && send_options)
@@ -1671,5 +1671,5 @@ sigterm_handler(int sig)		/* I - Signal */
 
 
 /*
- * End of "$Id: ipp.c 6484 2007-04-30 17:18:21Z mike $".
+ * End of "$Id: ipp.c 6583 2007-06-20 22:07:55Z mike $".
  */
