@@ -1,9 +1,9 @@
 /*
- * "$Id: lpr.c 5926 2006-09-05 20:45:47Z mike $"
+ * "$Id: lpr.c 6359 2007-03-19 14:46:09Z mike $"
  *
  *   "lpr" command for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2006 by Easy Software Products.
+ *   Copyright 1997-2007 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -232,7 +232,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 
 	      snprintf(email, sizeof(email), "mailto:%s@%s", cupsUser(),
 	               httpGetHostname(NULL, buffer, sizeof(buffer)));
-	      num_options = cupsAddOption("notify-recipient", email,
+	      num_options = cupsAddOption("notify-recipient-uri", email,
 	                                  num_options, &options);
 	    }
 	    break;
@@ -529,5 +529,5 @@ sighandler(int s)			/* I - Signal number */
 
 
 /*
- * End of "$Id: lpr.c 5926 2006-09-05 20:45:47Z mike $".
+ * End of "$Id: lpr.c 6359 2007-03-19 14:46:09Z mike $".
  */
