@@ -1,5 +1,5 @@
 /*
- * "$Id: adminutil.c 6362 2007-03-19 16:02:17Z mike $"
+ * "$Id: adminutil.c 6625 2007-07-06 22:39:54Z mike $"
  *
  *   Administration utility API definitions for the Common UNIX Printing
  *   System (CUPS).
@@ -656,12 +656,15 @@ cupsAdminExportSamba(
 				"are installed!")),
               sizeof(message));
 
-    _cupsSetError(IPP_INTERNAL_ERROR, message);
+    _cupsSetError(IPP_NOT_FOUND, message);
     _cupsLangPrintf(logfile, "%s\n", message);
   }
 
   if (have_drivers == 0)
+  {
+    _cupsSetError(IPP_NOT_FOUND, message);
     return (0);
+  }
 
  /*
   * Finally, associate the drivers we just added with the queue...
@@ -1260,14 +1263,21 @@ _cupsAdminSetServerSettings(
     {
       if (!wrote_browsing)
       {
+	int new_remote_printers = (remote_printers > 0 ||
+				   (remote_printers == -1 &&
+				    old_remote_printers > 0));
+	int new_share_printers = (share_printers > 0 ||
+				  (share_printers == -1 &&
+				   old_share_printers > 0));
+
         wrote_browsing = 1;
 
-        if (remote_printers > 0 || share_printers > 0)
+        if (new_remote_printers || new_share_printers)
 	{
-	  if (remote_printers > 0 && share_printers > 0)
+	  if (new_remote_printers && new_share_printers)
 	    cupsFilePuts(temp,
 	                 "# Enable printer sharing and shared printers.\n");
-	  else if (remote_printers > 0)
+	  else if (new_remote_printers)
 	    cupsFilePuts(temp,
 	                 "# Show shared printers on the local network.\n");
 	  else
@@ -1277,10 +1287,10 @@ _cupsAdminSetServerSettings(
 	  cupsFilePuts(temp, "Browsing On\n");
 	  cupsFilePuts(temp, "BrowseOrder allow,deny\n");
 
-	  if (remote_printers > 0)
+	  if (new_remote_printers)
 	    cupsFilePuts(temp, "BrowseAllow @LOCAL\n");
 
-	  if (share_printers > 0)
+	  if (new_share_printers)
 	    cupsFilePuts(temp, "BrowseAddress @LOCAL\n");
         }
 	else
@@ -2127,5 +2137,5 @@ write_option(cups_file_t     *dstfp,	/* I - PPD file */
 
 
 /*
- * End of "$Id: adminutil.c 6362 2007-03-19 16:02:17Z mike $".
+ * End of "$Id: adminutil.c 6625 2007-07-06 22:39:54Z mike $".
  */
