@@ -1,5 +1,5 @@
 /*
- * "$Id: ppd.c 6446 2007-04-04 23:43:50Z mike $"
+ * "$Id: ppd.c 6581 2007-06-20 21:17:49Z mike $"
  *
  *   PPD file routines for the Common UNIX Printing System (CUPS).
  *
@@ -1013,11 +1013,15 @@ ppdOpen2(cups_file_t *fp)		/* I - File to read from */
 
       if ((option = ppdFindOption(ppd, keyword + 6)) == NULL)
       {
+        int		groupidx;	/* Index for current group */
 	ppd_group_t	*gtemp;		/* Temporary group */
 
 
         DEBUG_printf(("%s option not found for %s...\n", keyword + 6, keyword));
 
+        if (group)
+          groupidx = group - ppd->groups; /* Save index for current group */
+
 	if ((gtemp = ppd_get_group(ppd, "General", _("General"), cg,
 	                           encoding)) == NULL)
 	{
@@ -1026,6 +1030,9 @@ ppdOpen2(cups_file_t *fp)		/* I - File to read from */
 	  goto error;
 	}
 
+        if (group)
+          group = ppd->groups + groupidx; /* Restore group pointer */
+
 	if ((option = ppd_get_option(gtemp, keyword + 6)) == NULL)
 	{
 	  DEBUG_printf(("Unable to get %s option!\n", keyword + 6));
@@ -1079,8 +1086,12 @@ ppdOpen2(cups_file_t *fp)		/* I - File to read from */
 
 	if ((option = ppdFindOption(ppd, "PageRegion")) == NULL)
 	{
+	  int		groupidx;	/* Index to current group */
 	  ppd_group_t	*gtemp;		/* Temporary group */
 
+          if (group)
+            groupidx = group - ppd->groups; /* Save index for current group */
+
 	  if ((gtemp = ppd_get_group(ppd, "General", _("General"), cg,
 				     encoding)) == NULL)
 	  {
@@ -1089,6 +1100,9 @@ ppdOpen2(cups_file_t *fp)		/* I - File to read from */
 	    goto error;
 	  }
 
+          if (group)
+            group = ppd->groups + groupidx; /* Restore group pointer */
+
 	  option = ppd_get_option(gtemp, "PageRegion");
         }
 
@@ -3102,5 +3116,5 @@ ppd_read(cups_file_t    *fp,		/* I - File to read from */
 
 
 /*
- * End of "$Id: ppd.c 6446 2007-04-04 23:43:50Z mike $".
+ * End of "$Id: ppd.c 6581 2007-06-20 21:17:49Z mike $".
  */
