@@ -1,5 +1,5 @@
 /*
- * "$Id: hpgl-input.c 5150 2006-02-22 19:21:50Z mike $"
+ * "$Id: hpgl-input.c 6625 2007-07-06 22:39:54Z mike $"
  *
  *   HP-GL/2 input processing for the Common UNIX Printing System (CUPS).
  *
@@ -182,7 +182,9 @@ ParseCommand(FILE    *fp,	/* I - File to read from */
   {
     bufptr = buf;
     while ((ch = getc(fp)) != ';')
-      if (bufptr < (buf + sizeof(buf) - 1))
+      if (ch == EOF)
+        break;
+      else if (bufptr < (buf + sizeof(buf) - 1))
         *bufptr++ = ch;
     *bufptr = '\0';
 
@@ -194,6 +196,10 @@ ParseCommand(FILE    *fp,	/* I - File to read from */
   while (!done)
     switch (ch = getc(fp))
     {
+      case EOF :
+          done = 1;
+          break;
+
       case ',' :
       case ' ' :
       case '\n' :
@@ -269,5 +275,5 @@ FreeParameters(int     num_params,	/* I - Number of parameters */
 
 
 /*
- * End of "$Id: hpgl-input.c 5150 2006-02-22 19:21:50Z mike $".
+ * End of "$Id: hpgl-input.c 6625 2007-07-06 22:39:54Z mike $".
  */
