@@ -1,5 +1,5 @@
 /*
- * "$Id: conf.c 6410 2007-03-28 18:03:19Z mike $"
+ * "$Id: conf.c 6550 2007-06-04 15:38:28Z mike $"
  *
  *   Configuration routines for the Common UNIX Printing System (CUPS).
  *
@@ -1745,7 +1745,7 @@ parse_aaa(cupsd_location_t *loc,	/* I - Location */
 
       cupsdLogMessage(CUPSD_LOG_WARN,
                       "\"AuthClass %s\" is deprecated; consider using "
-		      "\"Require @groupname\" on line %d.",
+		      "\"Require user @groupname\" on line %d.",
 	              value, linenum);
     }
     else if (!strcasecmp(value, "system"))
@@ -1756,7 +1756,7 @@ parse_aaa(cupsd_location_t *loc,	/* I - Location */
 
       cupsdLogMessage(CUPSD_LOG_WARN,
                       "\"AuthClass %s\" is deprecated; consider using "
-		      "\"Require @SYSTEM\" on line %d.",
+		      "\"Require user @SYSTEM\" on line %d.",
 	              value, linenum);
     }
     else
@@ -1773,7 +1773,7 @@ parse_aaa(cupsd_location_t *loc,	/* I - Location */
 
     cupsdLogMessage(CUPSD_LOG_WARN,
                     "\"AuthGroupName %s\" directive is deprecated; consider "
-		    "using \"Require @%s\" on line %d.",
+		    "using \"Require user @%s\" on line %d.",
 		    value, value, linenum);
   }
   else if (!strcasecmp(line, "Require"))
@@ -3290,5 +3290,5 @@ read_policy(cups_file_t *fp,		/* I - Configuration file */
 
 
 /*
- * End of "$Id: conf.c 6410 2007-03-28 18:03:19Z mike $".
+ * End of "$Id: conf.c 6550 2007-06-04 15:38:28Z mike $".
  */
