@@ -1,5 +1,5 @@
 /*
- * "$Id: cupsaddsmb.c 5926 2006-09-05 20:45:47Z mike $"
+ * "$Id: cupsaddsmb.c 6625 2007-07-06 22:39:54Z mike $"
  *
  *   "cupsaddsmb" command for the Common UNIX Printing System (CUPS).
  *
@@ -263,6 +263,9 @@ export_dest(http_t     *http,		/* I - Connection to server */
     status = cupsAdminExportSamba(dest, ppdfile, SAMBAServer,
                                   SAMBAUser, SAMBAPassword,
 				  Verbosity ? stderr : NULL);
+
+    if (!status && cupsLastError() == IPP_NOT_FOUND)
+      break;
   }
 
   unlink(ppdfile);
@@ -294,5 +297,5 @@ usage(void)
 
 
 /*
- * End of "$Id: cupsaddsmb.c 5926 2006-09-05 20:45:47Z mike $".
+ * End of "$Id: cupsaddsmb.c 6625 2007-07-06 22:39:54Z mike $".
  */
